package de.mdelab.mlsdm.diagram.custom.dialogs;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.RemoveCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.gmf.runtime.emf.commands.core.command.AbstractTransactionalCommand;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlexpressions.MLStringExpression;
import de.mdelab.mlexpressions.MlexpressionsFactory;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityFinalNode;
import de.mdelab.mlsdm.ActivityParameter;
import de.mdelab.mlsdm.ActivityParameterDirectionEnum;
import de.mdelab.mlsdm.MlsdmFactory;
import de.mdelab.mlsdm.OutputParameterValue;
import de.mdelab.mlsdm.diagram.edit.parts.ActivityFinalNode2EditPart;
import de.mdelab.mlsdm.diagram.edit.parts.ActivityFinalNodeEditPart;

/**
 * 
 * This dialog provides buttons to choose which Out Parameter should be edited.
 * It depends on information about the diagram and the stopNode which
 * expressions should be edited. It uses the EditExpressionDialog to edit the
 * expression. The mapping of expresssions and Out Parameters depends on the
 * order and number of expressions.
 * 
 * @author Patrick Rein (pre)
 */
public class ChooseActivityOutParameterDialog2 extends Dialog
{

	protected static final String		DIALOG_TITLE	= "Out Parameter";
	protected static final int			DIALOG_WIDTH	= 400;
	protected static final int			DIALOG_HEIGHT	= 400;
	protected static final int			DIALOG_PADDING	= 10;
	protected static final int			BUTTON_HEIGHT	= 35;
	protected String					ITEM_CONTAINER_TEXT		= "Out Parameters";
	protected Map<Button, Integer>		buttonDictionary;
	private ActivityFinalNode			finalNode;
	private Activity					activity;
	private TransactionalEditingDomain	changeCommandReceiver;
	private ActivityFinalNode2EditPart	editPart;
	private MlsdmEditOutParameterExpressionDialogNew	editExpressionDialog;
	
	private Composite 					composite;
	protected Group						itemContainer;
	
	private Composite[] parameterRows = new Composite[256];
	private OutputParameterValue[] parameters= new OutputParameterValue[256];
	private OutputParameterValue[] activityParameters= new OutputParameterValue[256];
	private int numberOfParameter = 0;

	public ChooseActivityOutParameterDialog2(Shell parent)
	{
		super(parent);
	}

	@Override
	public int open()
	{
		return super.open();
	}

	private List<OutputParameterValue> getOutParameters()
	{
		List<OutputParameterValue> resultParameters = new ArrayList<OutputParameterValue>();
		for (OutputParameterValue o: this.finalNode.getOutputParameterValues()){
			
			if (o.getParameter()!=null && !o.getParameter().getDirection().name().equals("IN")){
				resultParameters.add(o);
			}
		}
		return resultParameters;
	}
	protected void execute(AbstractTransactionalCommand command)
	{
		try
		{
			OperationHistoryFactory.getOperationHistory().execute(command, new NullProgressMonitor(), null);
		}
		catch (ExecutionException ee)
		{
			ee.printStackTrace();
		}
	}

	@Override
	public void create()
	{
		super.create();
	}

	@Override
	public boolean close()
	{
		return super.close();
	}

	@Override
	protected void okPressed()
	{
		setReturnCode(OK);
		this.close();
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#
	 *      createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent)
	{

		this.getShell().setText(DIALOG_TITLE);

		composite = (Composite) super.createDialogArea(parent);
		composite.setLayout(new RowLayout(SWT.VERTICAL | SWT.CENTER));
		composite.setSize(DIALOG_WIDTH, DIALOG_HEIGHT);
		
		itemContainer = new Group(composite, SWT.NONE);
		itemContainer.setText(ITEM_CONTAINER_TEXT);
		itemContainer.setLayout(new RowLayout(SWT.VERTICAL));
		
		buttonDictionary = new HashMap<Button, Integer>();
		addOutputParameterValueForFinalNode();

		this.numberOfParameter = this.getOutParameters().size();

		OutputParameterValue aParameter;
		for (int i = 0; i < this.getOutParameters().size(); i++)
		{
			aParameter = this.getOutParameters().get(i);
			if (!aParameter.getParameter().getDirection().name().equals("IN")){
				this.createActivityParameterRow(aParameter, i);
			}
		}
		editPart.addNotify();
		return composite;

	}
	
	protected void removeOldParameter(final OutputParameterValue aParameter, final Composite aParameterRow)
	{
		this.changeCommandReceiver.getCommandStack().execute(
				new RemoveCommand(this.editPart.getEditingDomain(), this.finalNode, this.finalNode.eClass().getEStructuralFeature(
						"outputParameterValues"), aParameter));
		aParameterRow.dispose();
		this.refreshVisually();
	}
	
	protected void removeOldActivityParameter(final OutputParameterValue aParam)
	{
		final EditingDomain domain = TransactionUtil.getEditingDomain(this.activity);
		domain.getCommandStack().execute(
				new RemoveCommand(this.editPart.getEditingDomain(), this.activity, this.activity.eClass()
						.getEStructuralFeature("parameters"), aParam));
		this.refreshVisually();
	}
	
	protected void writeOldActivityParameters(){
		for (OutputParameterValue aParam : this.activityParameters){
			removeOldActivityParameter(aParam);
		}
		for (OutputParameterValue aParam : this.activityParameters){
			addOldActivityParameter(aParam);
		}
	}

	protected void addOldParameter(OutputParameterValue aParameter, int number)
	{
		this.changeCommandReceiver.getCommandStack().execute(
				new AddCommand(this.editPart.getEditingDomain(), this.finalNode, this.finalNode.eClass().getEStructuralFeature(
						"outputParameterValues"), aParameter, numberOfParameter + 1));
		this.createActivityParameterRow(aParameter, number);
		this.refreshVisually();
	}

	protected void addOldActivityParameter(OutputParameterValue aParam)
	{
		final EditingDomain domain = TransactionUtil.getEditingDomain(this.activity);

		domain.getCommandStack().execute(
				new AddCommand(this.editPart.getEditingDomain(), this.activity, this.activity.eClass().getEStructuralFeature(
						"parameters"), aParam));

		//this.creatActivityParameterRow(aParameter, number);
		this.refreshVisually();
	}
	
	protected void editOutExpression(SelectionEvent e)
	{
		//EditExpressionDialog expressionDialog;
		int parameterIndex = this.buttonDictionary.get(e.getSource());
		//OutputParameterValue aParameter = this.activityParameters[parameterIndex];
		MlsdmEditOutParameterExpressionDialogNew dialog = new MlsdmEditOutParameterExpressionDialogNew(null);
		dialog.setActivity(this.activity);
		editExpressionDialog = dialog;
		editExpressionDialog.setExpression(this.getExpression(parameterIndex));
		//editExpressionDialog.setExpectedClassifier(aParameter.getParameter().getType());
		editExpressionDialog.setContextClassifier(null);
		editExpressionDialog.setExpressionOwner(this.finalNode);
		editExpressionDialog.setEditingDomain(this.editPart.getEditingDomain());
		editExpressionDialog.setStructuralFeature(this.finalNode.eClass().getEStructuralFeature("outputParameterValues"));

		dialog.open();
	}

	public void setChangeCommandReceiver(TransactionalEditingDomain changeCommandReceiver)
	{
		this.changeCommandReceiver = changeCommandReceiver;
	}

	public void setEditPart(ActivityFinalNode2EditPart stopNodeEP)
	{
		this.editPart = stopNodeEP;
	}

	protected MLExpression getExpression(int index)
	{
		EList<OutputParameterValue> el = this.finalNode.getOutputParameterValues();
		return (MLExpression) ((OutputParameterValue) el.get(index)).getExpression();
	}
	
	public void setNode(ActivityFinalNode model)
	{
		this.finalNode = model;
	}

	public void setActivity(Activity activity)
	{
		this.activity = activity;
	}
	
	private void addOutputParameterValueForFinalNode() {
		for (ActivityParameter aP : this.activity.getParameters()){
			boolean found = false;
			for (OutputParameterValue o: this.finalNode.getOutputParameterValues())
			{
				if (aP == o.getParameter()){
					found = true;
					if (aP.getDirection().equals(ActivityParameterDirectionEnum.IN)){
						ActivityFinalNode aFN = this.finalNode;
						EditingDomain domain2 = TransactionUtil.getEditingDomain(aFN);
						domain2.getCommandStack().execute(
								new RemoveCommand(this.editPart.getEditingDomain(), aFN, aFN.eClass().getEStructuralFeature(
										"outputParameterValues"), o));
					}
				}
			}
			if (!found && !aP.getDirection().equals(ActivityParameterDirectionEnum.IN)){
				OutputParameterValue o = MlsdmFactory.eINSTANCE.createOutputParameterValue();
				o.setParameter(aP);
				MLStringExpression sE = MlexpressionsFactory.eINSTANCE.createMLStringExpression();
				sE.setExpressionLanguageID("OCL");
				o.setExpression(sE);
				ActivityFinalNode aFN = this.finalNode;
				EditingDomain domain2 = TransactionUtil.getEditingDomain(aFN);
				domain2.getCommandStack().execute(
						new AddCommand(this.editPart.getEditingDomain(), aFN, aFN.eClass().getEStructuralFeature(
								"outputParameterValues"), o));
			}
		}
		addNotifyToAllActivityFinalNodes();

	}
	
	private void addNotifyToAllActivityFinalNodes(){
		for (Object o : editPart.getParent().getChildren()){
			if (o instanceof ActivityFinalNodeEditPart){
				((ActivityFinalNodeEditPart) o).addNotify();
			}
		}
	}

	protected void createActivityParameterRow(final OutputParameterValue aParameter, final int number)
	{
		/*String aString = "";

		parameterRows[number]= aParameterRow;
		parameters[number]= aParameter;*/
		String s;
		Button b;
		final Composite aParameterRow = new Composite(itemContainer, SWT.NONE);
		aParameterRow.setLayout(new RowLayout());
		parameterRows[number]= aParameterRow;
		parameters[number]= aParameter;
		if (aParameter != null && !aParameter.getParameter().getDirection().name().equals("IN"))/*&& aParameter.getEType() != null*/
		{
			s="->";
			//s = aParameter.getEType().getName();
			if (aParameter.getParameter()!=null)
				s = s + " " + aParameter.getParameter().getName();

			b = new Button(aParameterRow, SWT.PUSH);
			b.setLayoutData(new RowData(DIALOG_WIDTH/3*2, BUTTON_HEIGHT));
			b.setText(s);
			b.addSelectionListener(new SelectionListener()
			{

				@Override
				public void widgetSelected(SelectionEvent e)
				{
					editOutExpression(e);
				}

				@Override
				public void widgetDefaultSelected(SelectionEvent e)
				{
					widgetSelected(e);
				}

			});
			buttonDictionary.put(b, number);
		}
	}
	protected void refreshVisually()
	{
		((Composite) this.getDialogArea()).layout(true);
		this.getShell().pack();
		itemContainer.layout();
	}

}
