package de.mdelab.mlsdm.diagram.custom.edit.parts;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmf.runtime.draw2d.ui.figures.WrappingLabel;
import org.eclipse.gmf.runtime.notation.View;

import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityNodeContainer;
import de.mdelab.mlsdm.ActivityParameter;
import de.mdelab.mlsdm.InitialNode;
import de.mdelab.mlsdm.diagram.edit.parts.WrappingLabel13EditPart;

public class CustomInitialNodeSpecificationLabel2EditPart extends WrappingLabel13EditPart {
	
	public CustomInitialNodeSpecificationLabel2EditPart(View view)
	{
		super(view);
	}

	protected void updateFigure()
	{
		((WrappingLabel) getFigure()).setTextWrap(true);
		((WrappingLabel) getFigure()).setTextJustification(PositionConstants.CENTER);
		
		InitialNode initialNode = (InitialNode) (((View) getModel()).getElement());
		
		// DB: Issue #2091: Fix class cast exception when an initial node is dropped into a structured node 
		final ActivityNodeContainer actCont = initialNode.getActivityNodeContainer();
		//assert a != null;
		
		if ( actCont instanceof Activity ) {
			final Activity a = (Activity) actCont;
	
			if (!a.getParameters().isEmpty())
			{
				StringBuilder labelBuilder = new StringBuilder();
				
				this.removeListener();
				this.addListener();
				
				for (ActivityParameter param : a.getParameters())
				{
					switch (param.getDirection())
					{
						case IN:
							labelBuilder.append("in ");
							break;
						case OUT:
							labelBuilder.append("out ");
							break;
						case INOUT:
							labelBuilder.append("inout ");
							break;
					}
	
					labelBuilder.append(param.getName());
	
					labelBuilder.append(":");
	
					if (param.getType() != null)
					{
						labelBuilder.append(param.getType().getName());
					}
					else
					{
						labelBuilder.append("[NULL]");
					}
	
					labelBuilder.append(System.getProperty("line.separator"));
				}
	
				setLabelText(labelBuilder.toString());
			}
			else
			{
				setLabelText("");
			}
		}
	}
	
	private void addListener(){
		InitialNode initialNode = (InitialNode) ((View) getModel()).getElement();

		if (initialNode != null) {
	
			// DB: Issue #2091: Fix class cast exception when an initial node is dropped into a structured node 
			final ActivityNodeContainer actCont = initialNode.getActivityNodeContainer();
	
			if ( actCont instanceof Activity ) {
				final Activity a = (Activity) actCont;
			
				for (int i = 0;i <a.getParameters().size();i++) {
					addListenerFilter("Paramter" + i, this, (EObject) a.getParameters().toArray()[i]);
				}
			}
		}
	}

	private void removeListener(){
		InitialNode initialNode = (InitialNode) ((View) getModel()).getElement();

		if (initialNode != null) {

			// DB: Issue #2091: Fix class cast exception when an initial node is dropped into a structured node 
			final ActivityNodeContainer actCont = initialNode.getActivityNodeContainer();
			
			if ( actCont instanceof Activity ) {
				final Activity a = (Activity) actCont;

				for (int i = 0;i <a.getParameters().size()-1;i++) {
					removeListenerFilter("Paramter" + i);
				}
			}
		}
	}
	
	@Override
	protected void refreshVisuals()
	{
		super.refreshVisuals();
		updateFigure();
	}

	@Override
	protected void handleNotificationEvent(Notification event)
	{
		if (event.getNotifier() instanceof ActivityParameter)
		{
			updateFigure();
		}

		if (event.getNotifier() instanceof Activity)
		{
			updateFigure();
		}
		super.handleNotificationEvent(event);
	}
}
